# -*- coding: utf-8 -*-

from base import generate_answers_ini

import glob
import subprocess


base_files = [
    '.editorconfig',
    'requirements.txt',
    'buildout.cfg',
    'deployment.cfg',
    'bobtemplate.cfg',
    'haproxy.cfg',
    'services.cfg',
    'varnish.cfg',
    'webserver.cfg',
    'templates/apache_complex.tpl',
    'templates/apache_ssl.tpl',
    'templates/apache.tpl',
    'templates/haproxy.tpl',
    'templates/logrotate.tpl',
    'templates/nginx_complex.tpl',
    'templates/nginx_ssl_http2.tpl',
    'templates/nginx_ssl.tpl',
    'templates/nginx.tpl',
]


def test_cs_plone_buildout(tmpdir, capsys, config):
    template = """[variables]
package.description = Dummy package
package.example = True
package.git.init = True
package.plone.version = 5.1.2
package.buildout_zope_port_number = 8080

author.name = CodeSyntax
author.email = info@codesyntax.com
author.github.user = codesyntax

plone.version = {version}
""".format(
        version=config.version
    )
    generate_answers_ini(tmpdir.strpath, template)

    config.template = 'cs_plone_buildout'
    config.package_name = 'project1'

    result = subprocess.call(
        [
            'mrbob',
            '-O',
            config.package_name,
            'bobtemplates.cs:' + config.template,
            '--config',
            'answers.ini',
            '--non-interactive',
        ],
        cwd=tmpdir.strpath,
    )
    assert result == 0

    generated_files = glob.glob(
        tmpdir.strpath + '/' + config.package_name + '/*'
    )
    length = len(tmpdir.strpath + '/' + config.package_name + '/')
    generated_files = [f[length:] for f in generated_files]
    required_files = base_files
    assert required_files <= generated_files



# -*- coding: utf-8 -*-

import contextlib
import os


@contextlib.contextmanager
def dummy_contextmanager():
    yield None


def generate_answers_ini(path, template):
    with open(os.path.join(path, 'answers.ini'), 'w') as f:
        f.write(template)


def file_exists(base_path, file_path):
    is_file = os.path.isfile(base_path + file_path)
    return is_file



# -*- coding: utf-8 -*-

from bobtemplates.cs import buildout
from mrbob.configurator import Configurator


def test_prepare_renderer():
    configurator = Configurator(
        template='bobtemplates.cs:cs_plone_buildout',
        target_directory='collective.foo',
        variables={'buildout_zope_port_number': '8080'},
    )
    buildout.prepare_renderer(configurator)
    assert configurator.variables['template_id'] == 'cs_plone_buildout'


def test_post_renderer():
    buildout.post_renderer(None)



# -*- coding: utf-8 -*-

from base import dummy_contextmanager
from base import file_exists
from base import generate_answers_ini

import os.path
import subprocess


def test_behavior(tmpdir, capsys, config):
    template = """[variables]
package.description = Dummy package
package.example = True
package.git.init = True

author.name = The Plone Collective
author.email = collective@plone.org
author.github.user = collective

plone.version = {version}
""".format(
        version=config.version
    )
    generate_answers_ini(tmpdir.strpath, template)

    # generate template addon:
    config.template = 'addon'
    config.package_name = 'collective.task'
    result = subprocess.call(
        [
            'mrbob',
            '-O',
            config.package_name,
            'bobtemplates.plone:' + config.template,
            '--config',
            'answers.ini',
            '--non-interactive',
        ],
        cwd=tmpdir.strpath,
    )
    assert result == 0

    wd = os.path.abspath(os.path.join(tmpdir.strpath, config.package_name))

    # generate subtemplate content_type:
    #     template = """[variables]
    # """
    #     generate_answers_ini(wd, template)

    config.template = 'cs_migration'
    result = subprocess.call(
        [
            'mrbob',
            'bobtemplates.cs:' + config.template,
            '--config',
            'answers.ini',
            '--non-interactive',
        ],
        cwd=wd,
    )
    assert result == 0

    assert file_exists(wd, '/src/collective/task/blueprints/configure.zcml')
    assert file_exists(wd, '/src/collective/task/blueprints/pytz_fixer.py')
    assert file_exists(
        wd, '/src/collective/task/blueprints/reference_fields.py'
    )
    assert file_exists(wd, '/src/collective/task/pipelines/configure.zcml')
    assert file_exists(wd, '/src/collective/task/pipelines/migration1.cfg')

    with capsys.disabled() if config.verbose else dummy_contextmanager():
        setup_virtualenv_result = subprocess.call(['virtualenv', '.'], cwd=wd)
        assert setup_virtualenv_result == 0
        install_buildout_result = subprocess.call(
            ['./bin/pip', 'install', '-U', '-r', 'requirements.txt'], cwd=wd
        )
        assert install_buildout_result == 0
        annotate_result = subprocess.call(
            [
                'bin/buildout',
                'code-analysis:return-status-codes=True',
                'annotate',
            ],
            cwd=wd,
        )
        assert annotate_result == 0
        buildout_result = subprocess.call(
            ['bin/buildout', 'code-analysis:return-status-codes=True'], cwd=wd
        )
        assert buildout_result == 0
        test_result = subprocess.call(['bin/test'], cwd=wd)
        assert test_result == 0
        test__code_convention_result = subprocess.call(
            ['bin/code-analysis'], cwd=wd
        )
        assert test__code_convention_result == 0



# -*- coding: utf-8 -*-
"""
Original: https://gist.github.com/gyst/c3739160a18cf52c93e0132da511b2ee

see https://stackoverflow.com/questions/4008960/pytz-and-etc-gmt-5
for GMT+ and GMT- bug
"""
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import Condition
from collective.transmogrifier.utils import Matcher
from zope.interface import implementer
from zope.interface import provider

import logging


logger = logging.getLogger(__name__)


@provider(ISectionBlueprint)
@implementer(ISection)
class TimezoneFixerSection(object):
    """
    Fixes UnknownTimeZoneError: (UnknownTimeZoneError('GMT+1',),
    by replacing the (unknown) GMT+x timezone with Etc/GMT+x
    """


    def __init__(self, transmogrifier, name, options, previous):
        keys = options.get("keys") or "regexp:(.*[Dd]ate)$"
        self.keys = Matcher(*keys.splitlines())
        self.condition = Condition(
            options.get("condition", "python:True"), transmogrifier, name, options
        )
        self.previous = previous

    def __iter__(self):
        for item in self.previous:
            if self.condition(item):
                for key in item.keys():
                    match = self.keys(key)[1]
                    if match:
                        oldvalue = item[key]
                        if "GMT+" in item[key]:
                            item[key] = item[key].replace(" GMT+", " Etc/GMT-")  # noqa
                        elif "GMT-" in item[key]:
                            item[key] = item[key].replace(" GMT-", " Etc/GMT+")  # noqa
                        elif "GMT" in item[key]:
                            item[key] = item[key].replace(" GMT", " Etc/GMT")

                        newvalue = item[key]
                        if oldvalue != newvalue:
                            logger.info(
                                "pytz_fixer: field: {}, oldvalue {} newvalue {}".format(  # noqa
                                    key, oldvalue, newvalue
                                )
                            )
            yield item



# -*- coding: utf-8 -*-


def prepare_renderer(configurator):
    configurator.variables["template_id"] = "cs_plone_buildout"
    zope_port_number = int(configurator.variables["buildout_zope_port_number"])
    configurator.variables["zope_instance2_port_number"] = zope_port_number + 1
    configurator.variables["zope_instance3_port_number"] = zope_port_number + 2
    configurator.variables["zope_instance4_port_number"] = zope_port_number + 3
    configurator.variables["zope_instance5_port_number"] = zope_port_number + 4
    configurator.variables["zeo_port_number"] = zope_port_number + 10
    configurator.variables["haproxy_port_number"] = zope_port_number + 20
    configurator.variables["varnish_port_number"] = zope_port_number + 30


def post_renderer(configurator):
    """"""
    pass



# -*- coding: utf-8 -*-


class RegEntry(object):
    def __init__(self):
        self.template = ""
        self.plonecli_alias = ""
        self.depend_on = None
        self.deprecated = False
        self.info = ""


def cs_plone_buildout():
    reg = RegEntry()
    reg.template = "bobtemplates.cs:cs_plone_buildout"
    reg.plonecli_alias = "cs_plone_buildout"
    return reg


def cs_migration():
    reg = RegEntry()
    reg.template = "bobtemplates.cs:cs_migration"
    reg.plonecli_alias = "cs_migration"
    reg.depend_on = "plone_addon"
    return reg



# -*- coding: utf-8 -*-

from bobtemplates.cs import bobregistry


def test_reg_cs_plone_buildout():
    reg = bobregistry.cs_plone_buildout()
    assert reg.template == u'bobtemplates.cs:cs_plone_buildout'
    assert reg.plonecli_alias == u'cs_plone_buildout'
    assert not reg.depend_on
    assert not reg.deprecated
    assert not reg.info


def test_reg_cs_migration():
    reg = bobregistry.cs_migration()
    assert reg.template == u'bobtemplates.cs:cs_migration'
    assert reg.plonecli_alias == u'cs_migration'
    assert reg.depend_on == u'plone_addon'
    assert not reg.deprecated
    assert not reg.info



# -*- coding: utf-8 -*-

from bobtemplates.cs import base
from mrbob.configurator import Configurator

import os


def test_set_global_vars(tmpdir):
    template = """
[main]
version=5.1
"""
    target_dir = tmpdir.strpath + '/collective.foo'
    os.mkdir(target_dir)
    with open(os.path.join(target_dir + '/bobtemplate.cfg'), 'w') as f:
        f.write(template)
    configurator = Configurator(
        template='bobtemplates.cs:cs_plone_buildout',
        target_directory=target_dir,
        variables={'year': 1970, 'plone.version': '5.0-latest'},
    )
    base.set_global_vars(configurator)

    configurator = Configurator(
        template='bobtemplates.cs:cs_plone_buildout',
        target_directory=target_dir,
        variables={'year': 1970},
    )
    base.set_global_vars(configurator)



# -*- coding: utf-8 -*-
from bobtemplates.cs import migration
from mrbob.configurator import Configurator

import os


def test_prepare_renderer():
    configurator = Configurator(
        template='bobtemplates.cs:cs_migration', target_directory='.'
    )
    assert configurator
    migration.prepare_renderer(configurator)


def test_post_renderer(tmpdir):
    target_path = tmpdir.strpath + '/collective.foo'
    package_path = target_path + '/src/collective/foo'
    os.makedirs(target_path)
    os.makedirs(package_path)

    template = """
    dummy
    '-*- Extra requirements: -*-'
    """
    with open(os.path.join(target_path + '/setup.py'), 'w') as f:
        f.write(template)

    template = """
    <configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone">
    <!-- -*- extra stuff goes here -*- -->
    </configure>
    """
    with open(os.path.join(package_path + '/configure.zcml'), 'w') as f:
        f.write(template)

    configurator = Configurator(
        template='bobtemplates.cs:cs_migration',
        target_directory=package_path,
        bobconfig={'non_interactive': True},
    )
    migration.prepare_renderer(configurator)

    configurator.render()
    migration.post_renderer(configurator)



# -*- coding: utf-8 -*-

import os
import pytest


class Config(object):
    def __init__(self):
        self.version = None
        self.verbose = None
        self.template = None
        self.package_name = None
        self.subtemplates = []


@pytest.fixture(scope='module')
def config():
    config = Config()
    config.verbose = bool(os.environ.get('VERBOSE'))
    config.version = os.environ.get('VERSION', '5.1-latest')
    return config



# -*- coding: utf-8 -*-
from datetime import date

import logging


logger = logging.getLogger("bobtemplates.cs")


class SetupCfg(object):
    def __init__(self):
        self.version = None


def set_global_vars(configurator):
    configurator.variables["year"] = date.today().year
    version = configurator.variables.get("plone.version")
    _set_plone_version_variables(configurator, version)


def _set_plone_version_variables(configurator, version):
    version = configurator.variables.get("plone.version", version)
    if not version:
        return
    if "plone.is_plone5" not in configurator.variables:
        # Find out if it is supposed to be Plone 5.
        if version.startswith("5"):
            configurator.variables["plone.is_plone5"] = True
        else:
            configurator.variables["plone.is_plone5"] = False
    if "plone.minor_version" not in configurator.variables:
        # extract minor version (4.3)
        # (according to https://plone.org/support/version-support-policy)
        # this is used for the trove classifier in setup.py of the product
        configurator.variables["plone.minor_version"] = ".".join(version.split(".")[:2])
